from setuptools import setup

setup(name='elkdist',
      version='1.0',
      description='Probability distributions',
      packages=['elkdist'],
      author = 'Adham El-Khouly',
      zip_safe=False)
